@@ -1,5 +1,5 @@
 /* Determine the number of screen columns needed for a character.
-   Copyright (C) 2006, 2007, 2009, 2010 Free Software Foundation, Inc.
+   Copyright (C) 2006-2007, 2010-2011 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,54 +22,21 @@
 /* Get iswprint.  */
 #include <wctype.h>
 
-#if HAVE_LANGINFO_CODESET
-# include <langinfo.h>
-#endif
-
 #include "localcharset.h"
 #include "streq.h"
 #include "uniwidth.h"
 
-#undef wcwidth
-
-#include "uniwidth/cjk.h"
-
-static char cached_encoding[32];
-static int cached_is_cjk_encoding;
-static int cached_is_utf8_encoding;
-
-static const char *locale_charset_simple ()
-{
-#if HAVE_LANGINFO_CODESET
-  /* Most systems support nl_langinfo (CODESET) nowadays.  */
-  return nl_langinfo (CODESET);
-# else
-  /* Do the complex case */
-  return locale_charset ();
-# endif
-}
-
-static void cache_encoding ()
-{
-  const char *encoding = locale_charset_simple ();
-  if (!strncmp(encoding, cached_encoding, sizeof (cached_encoding)))
-    return;
-  strncpy (cached_encoding, encoding, sizeof (cached_encoding));
-  encoding = locale_charset ();
-  cached_is_utf8_encoding = STREQ (encoding, "UTF-8", 'U', 'T', 'F', '-', '8', 0, 0, 0 ,0);
-  cached_is_cjk_encoding = is_cjk_encoding (encoding);
-}
-
 int
-rpl_wcwidth (wchar_t wc)
+wcwidth (wchar_t wc)
+#undef wcwidth
 {
-  cache_encoding ();
   /* In UTF-8 locales, use a Unicode aware width function.  */
-  if (cached_is_utf8_encoding || cached_is_cjk_encoding)
+  const char *encoding = locale_charset ();
+  if (STREQ (encoding, "UTF-8", 'U', 'T', 'F', '-', '8', 0, 0, 0 ,0))
     {
       /* We assume that in a UTF-8 locale, a wide character is the same as a
          Unicode character.  */
-      return uc_width (wc, cached_is_cjk_encoding);
+      return uc_width1 (wc);
     }
   else
     {
