@@ -1,7 +1,7 @@
 /*
- * lftp and utils
+ * lftp - file transfer program
  *
- * Copyright (c) 1996-2010 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2011 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* $Id: CmdExec.h,v 1.76 2011/06/10 09:36:43 lav Exp $ */
@@ -116,6 +115,9 @@ private:
    CmdFeeder *feeder;
    bool feeder_called;
 
+   bool fed_at_finish;
+   void AtFinish();
+
    enum builtins
    {
       BUILTIN_NONE=0,
@@ -162,6 +164,7 @@ public:
    void FeedQuoted(const char *c);
    void Exit(int);
    void AtExit();
+   void AtExitBg();
    void EmptyCmds();
    bool WriteCmds(int fd) const;
    bool ReadCmds(int fd); // does not clear queue before reading (appends)
