@@ -129,6 +129,7 @@ void Http::MoveConnectionHere(Http *o)
 {
    send_buf=o->send_buf.borrow();
    recv_buf=o->recv_buf.borrow();
+   ssl=o->ssl.borrow();
    sock=o->sock; o->sock=-1;
    rate_limit=o->rate_limit.borrow();
    last_method=o->last_method; o->last_method=0;
@@ -146,6 +147,7 @@ void Http::Disconnect()
    if(sock!=-1)
    {
       LogNote(7,_("Closing HTTP connection"));
+      ssl=0;
       close(sock);
       sock=-1;
    }
