@@ -1,7 +1,7 @@
 /*
- * lftp and utils
+ * lftp - file transfer program
  *
- * Copyright (c) 2001-2006 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2011 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,14 +14,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
 #include "SMTask.h"
 #include "Timer.h"
 #include "xstring.h"
+#include "misc.h"
 
 #define now SMTask::now
 
@@ -53,10 +53,22 @@ void Timer::set_last_setting(const TimeInterval &i)
    infty_count+=IsInfty();
    re_set();
 }
+void Timer::add_random()
+{
+   if(random_max>0.0001) {
+      stop+=TimeDiff::valueOf(random_max*random01());
+   }
+}
 void Timer::re_set()
 {
    stop=start;
    stop+=last_setting;
+   add_random();
+   re_sort();
+}
+void Timer::AddRandom(double r) {
+   random_max=r;
+   add_random();
    re_sort();
 }
 void Timer::Set(const TimeInterval &i)
@@ -67,7 +79,7 @@ void Timer::Set(const TimeInterval &i)
 }
 void Timer::Reset(const Time &t)
 {
-   if(start>=t)
+   if(start>=t && stop>t)
       return;
    start=t;
    re_set();
@@ -106,6 +118,7 @@ void Timer::init()
    resource=0;
    closure=0;
    next_running=prev_running=0;
+   random_max=0;
    next_all=chain_all;
    chain_all=this;
 }
