@@ -1,11 +1,11 @@
 /*
- * lftp and utils
+ * lftp - file transfer program
  *
- * Copyright (c) 1996-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2011 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -14,11 +14,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: alias.cc,v 1.2.4.1 2002/01/15 16:20:23 lav Exp $ */
+/* $Id: alias.cc,v 1.8 2008/11/27 05:56:32 lav Exp $ */
 
 #include <config.h>
 
@@ -26,33 +25,18 @@
 #include "trio.h"
 
 #include "alias.h"
-#include "xmalloc.h"
 
 Alias *Alias::base;
 
-Alias::Alias(const char *alias,const char *value,Alias *next)
-{
-   this->alias=xstrdup(alias);
-   this->value=xstrdup(value);
-   this->next=next;
-}
-
-Alias::~Alias()
-{
-   xfree(this->alias);
-   xfree(this->value);
-}
-
 void Alias::Add(const char *alias,const char *value)
 {
    Alias **scan=&base;
    while(*scan)
    {
-      int dif=strcmp((*scan)->alias,alias);
+      int dif=strcasecmp((*scan)->alias,alias);
       if(dif==0)
       {
-	 xfree((*scan)->value);
-	 (*scan)->value=xstrdup(value);
+	 (*scan)->value.set(value);
 	 return;
       }
       if(dif>0)
@@ -67,7 +51,7 @@ void Alias::Del(const char *alias)
    Alias **scan=&base;
    while(*scan)
    {
-      int dif=strcmp((*scan)->alias,alias);
+      int dif=strcasecmp((*scan)->alias,alias);
       if(dif==0)
       {
 	 Alias *tmp=(*scan)->next;
@@ -84,7 +68,7 @@ const char *Alias::Find(const char *alias)
    Alias *scan=base;
    while(scan)
    {
-      int dif=strcmp(scan->alias,alias);
+      int dif=strcasecmp(scan->alias,alias);
       if(dif==0)
 	 return(scan->value);
       if(dif>0)
@@ -94,61 +78,36 @@ const char *Alias::Find(const char *alias)
    return 0;
 }
 
-#if 0
-void Alias::List()
-{
-   Alias *scan=base;
-   while(scan)
-   {
-      printf("alias %s \"%s\"\n",scan->alias,scan->value);
-      scan=scan->next;
-   }
-}
-#endif
-
 char *Alias::Format()
 {
-   char *res;
-   Alias *scan;
-   int size=0;
-
-   for(scan=base; scan; scan=scan->next)
+   xstring res;
+   for(Alias *scan=base; scan; scan=scan->next)
    {
-      size+=6+strlen(scan->alias)*2;
-      size+=2+strlen(scan->value)*2+1+1;
-   }
-   res=(char*)xmalloc(size+1);
-   char *store=res;
-
-   for(scan=base; scan; scan=scan->next)
-   {
-      strcpy(store,"alias ");
-      store+=strlen(store);
-      char *s=scan->alias;
+      res.append("alias ");
+      const char *s=scan->alias;
       while(*s)
       {
 	 if(strchr("\" \t\\>|",*s))
-	    *store++='\\';
-	 *store++=*s++;
+	    res.append('\\');
+	 res.append(*s++);
       }
-      *store++=' ';
+      res.append(' ');
       s=scan->value;
 
       bool par=false;
       if(*s==0 || strcspn(s," \t>|")!=strlen(s))
 	 par=true;
       if(par)
-	 *store++='"';
+	 res.append('"');
       while(*s)
       {
 	 if(strchr("\"\\",*s))
-	    *store++='\\';
-	 *store++=*s++;
+	    res.append('\\');
+	 res.append(*s++);
       }
       if(par)
-	 *store++='"';
-      *store++='\n';
+	 res.append('"');
+      res.append('\n');
    }
-   *store=0;
-   return res;
+   return res.borrow();
 }
