@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1999-2001 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2011 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,11 +14,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: lftp_rl.h,v 1.11 2008/11/27 05:56:39 lav Exp $ */
+/* $Id: lftp_rl.h,v 1.12 2009/11/17 08:18:37 lav Exp $ */
 
 #ifndef LFTP_RL_H
 #define LFTP_RL_H
@@ -42,6 +41,7 @@ void lftp_rl_bind(const char *key,const char *func);
 void lftp_rl_set_prompt(const char *p);
 void lftp_rl_write_history();
 void lftp_rl_read_history();
+void lftp_rl_history_stifle(int s);
 
 void lftp_rl_init(
    const char *readline_name,
