@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 2000-2006 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2011 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -27,11 +26,19 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <string.h>
 #include "lftp_ssl.h"
 #include "xmalloc.h"
 #include "ResMgr.h"
 #include "log.h"
 #include "misc.h"
+#include "network.h"
+#include "buffer.h"
+extern "C" {
+#include "c-ctype.h"
+#include "quotearg.h"
+#include "quote.h"
+}
 
 lftp_ssl_base::lftp_ssl_base(int fd1,handshake_mode_t m,const char *h)
    : hostname(h)
@@ -40,6 +47,7 @@ lftp_ssl_base::lftp_ssl_base(int fd1,handshake_mode_t m,const char *h)
    handshake_done=false;
    handshake_mode=m;
    fatal=false;
+   cert_error=false;
 }
 void lftp_ssl_base::set_error(const char *s1,const char *s2)
 {
@@ -57,6 +65,7 @@ void lftp_ssl_base::set_cert_error(const char *s)
    {
       set_error("Certificate verification",s);
       fatal=true;
+      cert_error=true;
    }
 }
 
@@ -260,6 +269,11 @@ lftp_ssl_gnutls::lftp_ssl_gnutls(int fd1,handshake_mode_t m,const char *h)
    gnutls_set_default_priority(session);
 
    gnutls_transport_set_ptr(session,(gnutls_transport_ptr_t)fd);
+
+   // hack for some ftp servers
+   const char *auth=ResMgr::Query("ftp:ssl-auth", hostname);
+   if(auth && !strncmp(auth, "SSL", 3))
+      gnutls_priority_set_direct(session, "NORMAL:+SSL3.0:-TLS1.0:-TLS1.1:-TLS1.2",0);
 }
 void lftp_ssl_gnutls::load_keys()
 {
@@ -337,10 +351,7 @@ void lftp_ssl_gnutls::verify_certificate_chain(const gnutls_datum_t *cert_chain,
     */
    bool check_hostname = ResMgr::QueryBool("ssl:check-hostname", hostname);
    if(check_hostname && !gnutls_x509_crt_check_hostname(cert[0], hostname))
-   {
-      xstring_ca err(xasprintf("The certificate's owner does not match hostname '%s'\n",hostname.get()));
-      set_cert_error(err);
-   }
+      set_cert_error(xstring::format("certificate common name doesn't match requested host name %s",quote(hostname)));
 
    for (i = 0; i < cert_chain_length; i++)
       gnutls_x509_crt_deinit(cert[i]);
@@ -750,7 +761,8 @@ lftp_ssl_openssl_instance::lftp_ssl_openssl_instance()
 #else
    SSLeay_add_ssl_algorithms();
    ssl_ctx=SSL_CTX_new(SSLv23_client_method());
-   SSL_CTX_set_options(ssl_ctx, SSL_OP_ALL|SSL_OP_NO_TICKET);
+   SSL_CTX_set_options(ssl_ctx, SSL_OP_ALL|SSL_OP_NO_TICKET|SSL_OP_NO_SSLv2);
+   SSL_CTX_set_cipher_list(ssl_ctx, "ALL:!aNULL:!eNULL:!SSLv2:!LOW:!EXP:!MD5:@STRENGTH");
    SSL_CTX_set_verify(ssl_ctx,SSL_VERIFY_PEER,lftp_ssl_openssl::verify_callback);
 //    SSL_CTX_set_default_passwd_cb(ssl_ctx,lftp_ssl_passwd_callback);
 
@@ -846,12 +858,12 @@ lftp_ssl_openssl::~lftp_ssl_openssl()
    SSL_free(ssl);
 }
 
-static const char *host;
+static const char *verify_callback_host;
 static int lftp_ssl_connect(SSL *ssl,const char *h)
 {
-   host=h;
+   verify_callback_host=h;
    int res=SSL_connect(ssl);
-   host=0;
+   verify_callback_host=0;
    return res;
 }
 bool lftp_ssl_openssl::check_fatal(int res)
@@ -885,11 +897,14 @@ int lftp_ssl_openssl::do_handshake()
       }
    }
    handshake_done=true;
+   check_certificate();
    SMTask::current->Timeout(0);
    return DONE;
 }
 int lftp_ssl_openssl::read(char *buf,int size)
 {
+   if(error)
+      return ERROR;
    int res=do_handshake();
    if(res!=DONE)
       return res;
@@ -912,6 +927,8 @@ int lftp_ssl_openssl::read(char *buf,int size)
 }
 int lftp_ssl_openssl::write(const char *buf,int size)
 {
+   if(error)
+      return ERROR;
    int res=do_handshake();
    if(res!=DONE)
       return res;
@@ -1099,6 +1116,425 @@ int lftp_ssl_openssl::verify_crl(X509_STORE_CTX *ctx)
     return 1;
 }
 
+static bool convert_from_utf8(char *str,int len)
+{
+   DirectedBuffer translate(DirectedBuffer::GET);
+   translate.SetTranslation("UTF-8",false);
+   translate.PutTranslated(str,len);
+   const char *str1,*str2;
+   int len1,len2;
+   translate.Get(&str1,&len1);
+   if(len1>len)
+      return false;  // no room to store expanded string
+
+   // be safe and try to convert back to UTF-8
+   DirectedBuffer translate_back(DirectedBuffer::PUT);
+   translate_back.SetTranslation("UTF-8",false);
+   translate_back.PutTranslated(str1,len1);
+   translate_back.Get(&str2,&len2);
+   if(len2!=len || memcmp(str2,str,len))
+      return false;  // conversion error
+
+   memcpy(str,str1,len1);
+   str[len1]=0;
+   return true;
+}
+
+/* begin curl-7.21.3 code */
+/* Copyright (c) 1998 - 2010, Daniel Stenberg, <daniel@haxx.se> */
+#define Curl_raw_toupper c_toupper
+#define Curl_raw_equal !strcmp
+#define Curl_inet_pton inet_pton
+#if INET6
+# define ENABLE_IPV6 1
+#endif
+/*
+ * Match a hostname against a wildcard pattern.
+ * E.g.
+ *  "foo.host.com" matches "*.host.com".
+ *
+ * We are a bit more liberal than RFC2818 describes in that we
+ * accept multiple "*" in pattern (similar to what some other browsers do).
+ * E.g.
+ *  "abc.def.domain.com" should strickly not match "*.domain.com", but we
+ *  don't consider "." to be important in CERT checking.
+ */
+#define HOST_NOMATCH 0
+#define HOST_MATCH   1
+
+static int hostmatch(const char *hostname, const char *pattern)
+{
+  for(;;) {
+    char c = *pattern++;
+
+    if(c == '\0')
+      return (*hostname ? HOST_NOMATCH : HOST_MATCH);
+
+    if(c == '*') {
+      c = *pattern;
+      if(c == '\0')      /* "*\0" matches anything remaining */
+        return HOST_MATCH;
+
+      while(*hostname) {
+        /* The only recursive function in libcurl! */
+        if(hostmatch(hostname++,pattern) == HOST_MATCH)
+          return HOST_MATCH;
+      }
+      break;
+    }
+
+    if(Curl_raw_toupper(c) != Curl_raw_toupper(*hostname++))
+      break;
+  }
+  return HOST_NOMATCH;
+}
+
+static int
+cert_hostcheck(const char *match_pattern, const char *hostname)
+{
+  if(!match_pattern || !*match_pattern ||
+      !hostname || !*hostname) /* sanity check */
+    return 0;
+
+  if(Curl_raw_equal(hostname, match_pattern)) /* trivial case */
+    return 1;
+
+  if(hostmatch(hostname,match_pattern) == HOST_MATCH)
+    return 1;
+  return 0;
+}
+
+/* Quote from RFC2818 section 3.1 "Server Identity"
+
+   If a subjectAltName extension of type dNSName is present, that MUST
+   be used as the identity. Otherwise, the (most specific) Common Name
+   field in the Subject field of the certificate MUST be used. Although
+   the use of the Common Name is existing practice, it is deprecated and
+   Certification Authorities are encouraged to use the dNSName instead.
+
+   Matching is performed using the matching rules specified by
+   [RFC2459].  If more than one identity of a given type is present in
+   the certificate (e.g., more than one dNSName name, a match in any one
+   of the set is considered acceptable.) Names may contain the wildcard
+   character * which is considered to match any single domain name
+   component or component fragment. E.g., *.a.com matches foo.a.com but
+   not bar.foo.a.com. f*.com matches foo.com but not bar.com.
+
+   In some cases, the URI is specified as an IP address rather than a
+   hostname. In this case, the iPAddress subjectAltName must be present
+   in the certificate and must exactly match the IP in the URI.
+
+*/
+void lftp_ssl_openssl::check_certificate()
+{
+  X509 *server_cert = SSL_get_peer_certificate (ssl);
+  if (!server_cert)
+    {
+      set_cert_error(xstring::format(_("No certificate presented by %s.\n"),
+                 quotearg_style (escape_quoting_style, hostname)));
+      return;
+    }
+
+  int matched = -1; /* -1 is no alternative match yet, 1 means match and 0
+                       means mismatch */
+  int target = GEN_DNS; /* target type, GEN_DNS or GEN_IPADD */
+  size_t addrlen = 0;
+  STACK_OF(GENERAL_NAME) *altnames;
+#ifdef ENABLE_IPV6
+  struct in6_addr addr;
+#else
+  struct in_addr addr;
+#endif
+
+  sockaddr_u fd_addr;
+  socklen_t fd_addr_len = sizeof(fd_addr);
+  getsockname(fd,&fd_addr.sa,&fd_addr_len);
+
+#ifdef ENABLE_IPV6
+  if(fd_addr.sa.sa_family==AF_INET6 &&
+     Curl_inet_pton(AF_INET6, hostname, &addr)) {
+    target = GEN_IPADD;
+    addrlen = sizeof(struct in6_addr);
+  }
+  else
+#endif
+    if(Curl_inet_pton(AF_INET, hostname, &addr)) {
+      target = GEN_IPADD;
+      addrlen = sizeof(struct in_addr);
+    }
+
+  /* get a "list" of alternative names */
+  altnames = (STACK_OF(GENERAL_NAME)*)X509_get_ext_d2i(server_cert, NID_subject_alt_name, NULL, NULL);
+
+  if(altnames) {
+    int numalts;
+    int i;
+
+    /* get amount of alternatives, RFC2459 claims there MUST be at least
+       one, but we don't depend on it... */
+    numalts = sk_GENERAL_NAME_num(altnames);
+
+    /* loop through all alternatives while none has matched */
+    for (i=0; (i<numalts) && (matched != 1); i++) {
+      /* get a handle to alternative name number i */
+      const GENERAL_NAME *check = sk_GENERAL_NAME_value(altnames, i);
+
+      /* only check alternatives of the same type the target is */
+      if(check->type == target) {
+        /* get data and length */
+        const char *altptr = (char *)ASN1_STRING_data(check->d.ia5);
+        size_t altlen = (size_t) ASN1_STRING_length(check->d.ia5);
+
+        switch(target) {
+        case GEN_DNS: /* name/pattern comparison */
+          /* The OpenSSL man page explicitly says: "In general it cannot be
+             assumed that the data returned by ASN1_STRING_data() is null
+             terminated or does not contain embedded nulls." But also that
+             "The actual format of the data will depend on the actual string
+             type itself: for example for and IA5String the data will be ASCII"
+
+             Gisle researched the OpenSSL sources:
+             "I checked the 0.9.6 and 0.9.8 sources before my patch and
+             it always 0-terminates an IA5String."
+          */
+          if((altlen == strlen(altptr)) &&
+             /* if this isn't true, there was an embedded zero in the name
+                string and we cannot match it. */
+             cert_hostcheck(altptr, hostname))
+            matched = 1;
+          else
+            matched = 0;
+          break;
+
+        case GEN_IPADD: /* IP address comparison */
+          /* compare alternative IP address if the data chunk is the same size
+             our server IP address is */
+          if((altlen == addrlen) && !memcmp(altptr, &addr, altlen))
+            matched = 1;
+          else
+            matched = 0;
+          break;
+        }
+      }
+    }
+    GENERAL_NAMES_free(altnames);
+  }
+
+  if(matched == 1)
+    /* an alternative name matched the server hostname */
+    Log::global->Format(9, "Certificate verification: subjectAltName: %s matched\n", quote(hostname));
+  else if(matched == 0) {
+    /* an alternative name field existed, but didn't match and then
+       we MUST fail */
+    set_cert_error(xstring::format("subjectAltName does not match %s", quote(hostname)));
+  }
+  else {
+    /* we have to look to the last occurence of a commonName in the
+       distinguished one to get the most significant one. */
+    int j,i=-1 ;
+
+/* The following is done because of a bug in 0.9.6b */
+
+    unsigned char *nulstr = (unsigned char *)"";
+    unsigned char *peer_CN = nulstr;
+
+    X509_NAME *name = X509_get_subject_name(server_cert) ;
+    if(name)
+      while((j = X509_NAME_get_index_by_NID(name, NID_commonName, i))>=0)
+        i=j;
+
+    /* we have the name entry and we will now convert this to a string
+       that we can use for comparison. Doing this we support BMPstring,
+       UTF8 etc. */
+
+    if(i>=0) {
+      ASN1_STRING *tmp = X509_NAME_ENTRY_get_data(X509_NAME_get_entry(name,i));
+
+      /* In OpenSSL 0.9.7d and earlier, ASN1_STRING_to_UTF8 fails if the input
+         is already UTF-8 encoded. We check for this case and copy the raw
+         string manually to avoid the problem. This code can be made
+         conditional in the future when OpenSSL has been fixed. Work-around
+         brought by Alexis S. L. Carvalho. */
+      if(tmp) {
+        if(ASN1_STRING_type(tmp) == V_ASN1_UTF8STRING) {
+          j = ASN1_STRING_length(tmp);
+          if(j >= 0) {
+            peer_CN = (unsigned char*)OPENSSL_malloc(j+1);
+            if(peer_CN) {
+              memcpy(peer_CN, ASN1_STRING_data(tmp), j);
+              peer_CN[j] = '\0';
+            }
+          }
+        }
+        else /* not a UTF8 name */
+          j = ASN1_STRING_to_UTF8(&peer_CN, tmp);
+
+        if(peer_CN && ((int)strlen((char *)peer_CN) != j)) {
+          /* there was a terminating zero before the end of string, this
+             cannot match and we return failure! */
+          set_cert_error("illegal cert name field (contains NUL character)");
+        }
+      }
+    }
+
+    if(peer_CN == nulstr)
+       peer_CN = NULL;
+    else {
+      /* convert peer_CN from UTF8 */
+      if(!convert_from_utf8((char*)peer_CN, strlen((char*)peer_CN)))
+	 set_cert_error("invalid cert name field (cannot convert from UTF8)");
+    }
+
+    if(cert_error)
+      /* error already detected, pass through */
+      ;
+    else if(!peer_CN) {
+      set_cert_error("unable to obtain common name from peer certificate");
+    }
+    else if(!cert_hostcheck((const char *)peer_CN, hostname)) {
+        set_cert_error(xstring::format("certificate subject name %s does not match "
+              "target host name %s", quote_n(0,(const char *)peer_CN), quote_n(1,hostname)));
+    }
+    else {
+      Log::global->Format(9, "Certificate verification: common name: %s matched\n", quote((char*)peer_CN));
+    }
+    if(peer_CN)
+      OPENSSL_free(peer_CN);
+  }
+}
+/* end curl code */
+
+/* begin wget-1.12 code */
+#if 0
+#define ASTERISK_EXCLUDES_DOT   /* mandated by rfc2818 */
+
+/* Return true is STRING (case-insensitively) matches PATTERN, false
+   otherwise.  The recognized wildcard character is "*", which matches
+   any character in STRING except ".".  Any number of the "*" wildcard
+   may be present in the pattern.
+
+   This is used to match of hosts as indicated in rfc2818: "Names may
+   contain the wildcard character * which is considered to match any
+   single domain name component or component fragment. E.g., *.a.com
+   matches foo.a.com but not bar.foo.a.com. f*.com matches foo.com but
+   not bar.com [or foo.bar.com]."
+
+   If the pattern contain no wildcards, pattern_match(a, b) is
+   equivalent to !strcasecmp(a, b).  */
+
+static bool
+pattern_match (const char *pattern, const char *string)
+{
+  const char *p = pattern, *n = string;
+  char c;
+  for (; (c = c_tolower (*p++)) != '\0'; n++)
+    if (c == '*')
+      {
+        for (c = c_tolower (*p); c == '*'; c = c_tolower (*++p))
+          ;
+        for (; *n != '\0'; n++)
+          if (c_tolower (*n) == c && pattern_match (p, n))
+            return true;
+#ifdef ASTERISK_EXCLUDES_DOT
+          else if (*n == '.')
+            return false;
+#endif
+        return c == '\0';
+      }
+    else
+      {
+        if (c != c_tolower (*n))
+          return false;
+      }
+  return *n == '\0';
+}
+
+/* Verify the validity of the certificate presented by the server.
+   Also check that the "common name" of the server, as presented by
+   its certificate, corresponds to HOST.  (HOST typically comes from
+   the URL and is what the user thinks he's connecting to.)
+
+   This assumes that ssl_connect_wget has successfully finished, i.e. that
+   the SSL handshake has been performed and that FD is connected to an
+   SSL handle.
+
+   If opt.check_cert is true (the default), this returns 1 if the
+   certificate is valid, 0 otherwise.  If opt.check_cert is 0, the
+   function always returns 1, but should still be called because it
+   warns the user about any problems with the certificate.  */
+
+void lftp_ssl_openssl::check_certificate_wget ()
+{
+  X509 *cert;
+  char common_name[256];
+
+  cert = SSL_get_peer_certificate (ssl);
+  if (!cert)
+    {
+      set_cert_error(xstring::format(_("No certificate presented by %s.\n"),
+                 quotearg_style (escape_quoting_style, hostname)));
+      return;
+    }
+
+  /* LAV: certificate validity is already checked in callback. */
+
+  /* Check that HOST matches the common name in the certificate.
+     #### The following remains to be done:
+
+     - It should use dNSName/ipAddress subjectAltName extensions if
+       available; according to rfc2818: "If a subjectAltName extension
+       of type dNSName is present, that MUST be used as the identity."
+
+     - When matching against common names, it should loop over all
+       common names and choose the most specific one, i.e. the last
+       one, not the first one, which the current code picks.
+
+     - Ensure that ASN1 strings from the certificate are encoded as
+       UTF-8 which can be meaningfully compared to HOST.  */
+
+  X509_NAME *xname = X509_get_subject_name(cert);
+  common_name[0] = '\0';
+  X509_NAME_get_text_by_NID (xname, NID_commonName, common_name,
+                             sizeof (common_name));
+
+  if (!pattern_match (common_name, hostname))
+    {
+      set_cert_error(xstring::format(_("certificate common name %s doesn't match requested host name %s"),
+                 quote_n (0, common_name), quote_n (1, hostname)));
+    }
+  else
+    {
+      /* We now determine the length of the ASN1 string. If it differs from
+       * common_name's length, then there is a \0 before the string terminates.
+       * This can be an instance of a null-prefix attack.
+       *
+       * https://www.blackhat.com/html/bh-usa-09/bh-usa-09-archives.html#Marlinspike
+       * */
+
+      int i = -1, j;
+      X509_NAME_ENTRY *xentry;
+      ASN1_STRING *sdata;
+
+      if (xname) {
+        for (;;)
+          {
+            j = X509_NAME_get_index_by_NID (xname, NID_commonName, i);
+            if (j == -1) break;
+            i = j;
+          }
+      }
+
+      xentry = X509_NAME_get_entry(xname,i);
+      sdata = X509_NAME_ENTRY_get_data(xentry);
+      if (strlen (common_name) != (size_t)ASN1_STRING_length (sdata))
+          set_cert_error(_("certificate common name is invalid (contains a NUL character)"));
+    }
+
+  X509_free (cert);
+}
+#endif
+/* end wget code */
+
 int lftp_ssl_openssl::verify_callback(int ok,X509_STORE_CTX *ctx)
 {
    static X509 *prev_cert=0;
@@ -1122,7 +1558,7 @@ int lftp_ssl_openssl::verify_callback(int ok,X509_STORE_CTX *ctx)
 
    int error=X509_STORE_CTX_get_error(ctx);
 
-   bool verify=ResMgr::QueryBool("ssl:verify-certificate",host);
+   bool verify=ResMgr::QueryBool("ssl:verify-certificate",verify_callback_host);
 
    if(!ok)
    {
