@@ -1,7 +1,7 @@
 /*
- * lftp and utils
+ * lftp - file transfer program
  *
- * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2011 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,11 +14,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: misc.h,v 1.51 2010/02/05 17:59:50 lav Exp $ */
+/* $Id: misc.h,v 1.55 2011/03/25 14:35:43 lav Exp $ */
 
 #ifndef MISC_H
 #define MISC_H
@@ -102,10 +101,6 @@ void xgettimeofday(time_t *sec, int *usec);
 /* returns malloc'd date */
 char *xstrftime(const char *format, const struct tm *tm);
 
-/* returns malloc'd formatted string */
-char *xvasprintf(const char *format, va_list ap);
-char *xasprintf(const char *format, ...);
-
 void strip_trailing_slashes(xstring& fn);
 xstring& dirname_modify(xstring& fn);
 xstring& dirname(const char *path);  // returns a tmp
@@ -125,8 +120,12 @@ static inline char *memrchr(char *buf,char c,size_t len) {
    return const_cast<char*>(memrchr(const_cast<const char*>(buf),c,len));
 }
 
-const char *shell_encode(const char *);
+const xstring& shell_encode(const char *);
 void remove_tags(char *buf);
+void rtrim(char *s);
+
+void random_init();
+double random01();
 
 #define ListAdd(type,chain,this,next)  \
 do {				       \
